![Vivid](http://yuao.github.io/Vivid/assets/logo.png)

![CocoaPods Platform](https://img.shields.io/cocoapods/p/Vivid.svg?style=flat-square)
![CocoaPods Version](https://img.shields.io/cocoapods/v/Vivid.svg?style=flat-square)
![CocoaPods License](https://img.shields.io/cocoapods/l/Vivid.svg?style=flat-square)

A set of filters and utilities for Apple's [Core Image](https://developer.apple.com/library/mac/documentation/GraphicsImaging/Conceptual/CoreImaging/ci_intro/ci_intro.html) framework.

Available on both OS X and iOS.

Involving...

##Core Image Filters

###Filters

####YUCIRGBToneCurve

Adjusts tone response of the R, G, and B channels of an image.

The filter takes in an array of control points that define the spline curve for each color component, or for all three in the composite.

These are stored as `CIVector`s in an `NSArray`, with normalized X and Y coordinates from `0` to `1`.

The defaults are `[(0,0), (0.5,0.5), (1,1)]`

![Preview](http://yuao.github.io/Vivid/previews/YUCIRGBToneCurve.png)

####YUCIColorLookup

Uses a color lookup table (LUT) to remap the colors in an image. The default LUT can be found at `Sources/YUCIColorLookupTableDefault.png`

*This filter may not work well in the default light-linear working color space. Use `kCIContextWorkingColorSpace` key to specify a working color space when creating the `CIContext` object.*

![Preview](http://yuao.github.io/Vivid/previews/YUCIColorLookup.png)

####YUCISurfaceBlur

A bilateral filter. Blurs an image while preserving edges. This filter is almost identical to Photoshop's "Surface Blur" filter.

Useful for creating special effects and for removing noise or graininess. Slow on large `inputRadius`.

![Preview](http://yuao.github.io/Vivid/previews/YUCISurfaceBlur.png)

####YUCITriangularPixellate

Maps an image to colored triangles.

![Preview](http://yuao.github.io/Vivid/previews/YUCITriangularPixellate.png)

####YUCIFXAA

A basic implementation of FXAA (Fast Approximate Anti-Aliasing).

![Preview](http://yuao.github.io/Vivid/previews/YUCIFXAA.png)

####YUCIHistogramEqualization

Perform a [Histogram Equalization](https://en.wikipedia.org/wiki/Histogram_equalization) on the input image. Internally uses `Accelerate.framework`.

![Preview](http://yuao.github.io/Vivid/previews/YUCIHistogramEqualization.png)

####YUCIReflectedTile

Produces a tiled image from a source image by reflecting pixels over the edges.

![Preview](http://yuao.github.io/Vivid/previews/YUCIReflectedTile.png)

####YUCICLAHE

Perform a [Contrast Limited Adaptive Histogram Equalization](https://en.wikipedia.org/wiki/Adaptive_histogram_equalization#Contrast_Limited_AHE) on the lightness channel of the input image. 

![Preview](http://yuao.github.io/Vivid/previews/YUCICLAHE.png)

###Transitions

####YUCICrossZoomTransition

A transition that pushes the `inputImage` toward the viewer and then snaps back with the `inputTargetImage`.

![Preview](http://yuao.github.io/Vivid/previews/YUCICrossZoomTransition.gif)

####YUCIFlashTransition

Transitions from one image to another by creating a flash effect.

![Preview](http://yuao.github.io/Vivid/previews/YUCIFlashTransition.gif)

###Generators

####YUCIStarfieldGenerator

Generate a starfield image. Animatable by changing the `inputTime` parameter. Based on [Star Nest](https://www.shadertoy.com/view/XlfGRj) by Pablo Román Andrioli

![Preview](http://yuao.github.io/Vivid/previews/YUCIStarfieldGenerator.gif)

####YUCIBlobsGenerator

Generate a image with colorful blobs. Animatable by changing the `inputTime` parameter. Based on [Blobs](https://www.shadertoy.com/view/lsfGzr) by [@paulofalcao](https://twitter.com/paulofalcao)

![Preview](http://yuao.github.io/Vivid/previews/YUCIBlobsGenerator.gif)

##Utilities

####YUCIFilterConstructor

A singleton that conforms to `CIFilterConstructor` protocol.

Can be used in `+[CIFilter registerFilterName:constructor:classAttributes:]` to register a `CIFilter`. This filter constructor simply assume that the `filterName` is the class name of the custom `CIFilter` and calls `[[FilterClass alloc] init]` to construct a filter.

####YUCIFilterPreviewGenerator

Can be used to generate a preview image/gif for a filter. All the preview images/gifs on this page are generated by this utility. __For demonstration/testing purposes only, do not use it in your production code.__

##Next

- [x] Add filter previews to readme.
- [x] AA for triangular pixellate filter.
- [x] CLAHE
- [ ] Write a paper on the implementation of CLAHE.
- [ ] Kuwahara filter

##Based on Vivid

####[YUCIHighPassSkinSmoothing](https://github.com/YuAo/YUCIHighPassSkinSmoothing)

An implementation of High Pass Skin Smoothing
